#ifndef __BIAWEBSIDEBAR__
#define __BIAWEBSIDEBAR__
#include <string>
#include <list>
#include <iostream>
#include <fstream>
#include <fmt/format.h>
#include "biawebutil.hpp"

// classes to describe the a list of (link) items and sidebar containers which form part of 
// main document
namespace biaweb {
    // class to represent a generic link item which can contain a text and link or only
    // text
    class GenericLinkItem {
      protected:
        // sidebar text and url
        std::string itemtext;
        std::string itemurl;
      public:
        std::string get_item_text () {
            return this->itemtext;
        }
        void set_item_text (std::string text) {
            this->itemtext = escape_html (text);
        }
        std::string get_item_url () {
            return this->itemurl;
        }
        void set_item_url (std::string url) {
            this->itemurl = escape_html (url);
        }
        // output to HTML using a template directory specified
        std::string to_html (std::string templatedir);
    
        GenericLinkItem (std::string text = "", std::string url = "") {
            this->itemtext = escape_html (text);
            this->itemurl = escape_html (url);
        }
    };

    std::string GenericLinkItem::to_html (std::string templatedir) {
        std::string html = "";
        // if url is not empty it is a link item so load the sidebar link template
        if (! this->itemurl.empty ()) {
            if (!this->itemtext.empty ())
            {
                std::ifstream tpl_linkitem (templatedir + "/genericlinklistitem.tpl.html");

                std::string tpl_linkitem_str ( (std::istreambuf_iterator<char> (tpl_linkitem)),
                                                (std::istreambuf_iterator<char> ()));
                tpl_linkitem.close ();
                html = fmt::format (tpl_linkitem_str, 
                                        fmt::arg ("itemurl", this->itemurl),
                                        fmt::arg ("itemtext", this->itemtext));
            }
        }
        // Non link item. Load the normal sidebar item template.
        else if (! this->itemtext.empty ())
        {
            std::ifstream tpl_item (templatedir + "/genericlistitem.tpl.html");

            std::string tpl_item_str ( (std::istreambuf_iterator<char> (tpl_item)),
                                        (std::istreambuf_iterator<char> ()));
            tpl_item.close ();
            html = fmt::format (tpl_item_str, fmt::arg ("itemtext", this->itemtext) );
        } 
        return html;
    }
    
    // Class to represent a sidebar, which contains one heading and a list of items 
    // either links or non-link items.
    class SideBar {
      protected:
        std::string sidebar_title;
        std::list <GenericLinkItem> items;
        
      public:
        void set_title (std::string title) {
            this->sidebar_title = title;
        }

        void add_sidebar_item (GenericLinkItem item) {
            this->items.insert (this->items.cend(), item);
        }

        // render the sidebar using the template directory specified
        std::string to_html (std::string templatedir) ;
    };

    // render the sidebar to HTML representation from the template.
    std::string SideBar::to_html (std::string templatedir) {
        std::ifstream sidetpl (templatedir + "/sidebar.tpl.html");
        std::string sidetpl_str ( ( std::istreambuf_iterator<char> (sidetpl)) , 
                                       (std::istreambuf_iterator<char> ()));
        sidetpl.close ();
        std::string listitems;
        // first get the sidebar items and render them to HTML
        for (GenericLinkItem item : this->items) {
            listitems += item.to_html (templatedir);
        }
        
        std::string html = "";
        // if there are items, sidebar should be rendered otherwise not needed
        if (items.size () > 0)
            html = fmt::format (sidetpl_str, 
                                        fmt::arg ("title", this->sidebar_title),
                                        fmt::arg ("items", listitems));

        return html; 
    }
}

#endif
